/*
 * Created on 01.01.2004
 * 
 * $Id: AdminFacadeSessionBean.java,v 1.2 2006/10/19 20:23:16 hwr Exp $
 */
package de.bsd.weblog.ejb;

import java.util.Collection;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;
import javax.persistence.Query;

import org.apache.log4j.Logger;

import de.bsd.weblog.interfaces.AdminFacade;
import de.bsd.weblog.persistence.NRolle;
import de.bsd.weblog.persistence.PLevel;
import de.bsd.weblog.persistence.Poster;
import de.bsd.weblog.persistence.Weblog;

/**
 * Facade fr die Administration
 * @author hwr@pilhuhn.de
 * @version $Revision: 1.2 $
 */
@Stateless
@RolesAllowed("ADMIN")
public class AdminFacadeSessionBean extends FacadeSessionBean implements AdminFacade
{
	
	private static Logger log = Logger.getLogger("AdminFacade");

	@PersistenceContext
	private EntityManager em;
	
	/**
	 * Legt ein neues leeres Blog in der Datenbank an
	 * 
	 * @param name
	 * @return
	 */
	public boolean weblogAnlegen(String name) {

		try {
			Weblog w = new Weblog(name);
			em.persist(w);
			log.info("Weblog " + name + " angelegt");
		} catch (PersistenceException e) {
			// Name existiert schon in der Datenbank
			log.debug("Weblog " + name + " existierte schon");
			return false;
		}
		return true;
	}

	
	/* (non-Javadoc)
	 * @see de.bsd.weblog.ejb.AdminFacade#addPoster(java.lang.String, java.lang.String, java.lang.String)
	 */
	public boolean posterAnlegen(String eMail,String name,String password) {
		
		Poster p = posterNachName(name);
		if (p != null)
			return false; // poster exists .. nothing to do
		Poster pl = new Poster(eMail,name);   
		pl.setPasswort(password);
		pl.setRolle(NRolle.POSTER);
		pl.setLevel(PLevel.NORMAL);
		em.persist(pl);
		
		return true;
	}
	
	/* (non-Javadoc)
	 * @see de.bsd.weblog.ejb.AdminFacade#addPosterToWeblog(java.lang.String, java.lang.String)
	 */
	public boolean posterZuWeblog(String poster, String weblog) {
		
		Weblog wl = weblogNachName(weblog);
		Poster p = posterNachName(poster);
		if (log.isDebugEnabled()) {
			log.debug("posterZuWeblog: Blog: " + weblog + ", poster: " + poster);
			log.debug("   blog: " + wl + ", poster: " + p);
		}
		wl.getNutzer().add(p);
		em.merge(wl); 
		
		return true;
	}
	
	/* (non-Javadoc)
	 * @see de.bsd.weblog.ejb.AdminFacade#deleteWeblog(java.lang.String)
	 */
	public boolean weblogLoeschen(String weblog) {
		if (weblog==null || weblog.equals(""))
			return false;

		Weblog wl = weblogNachName(weblog);
		em.remove(wl);
		em.flush();
		
		return true;
	}


	/** 
	 * Listet alle registrierten Poster
	 * @see de.bsd.weblog.interfaces.AdminFacade#listeAllePoster()
	 */
	@SuppressWarnings("unchecked")
	public Collection<Poster> listeAllePoster() {
		
		Query q = em.createNamedQuery("Poster.PosterListe");
		Collection<Poster> ret = q.getResultList();
		return ret;
		
	}

	
}
