/*
 * $Id: FacadeSessionBean.java,v 1.4 2006/10/26 10:31:22 hwr Exp $
 */
package de.bsd.weblog.ejb;


import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import javax.annotation.security.PermitAll;
import javax.ejb.Stateless;
import javax.interceptor.Interceptors;
import javax.jws.WebMethod;
import javax.jws.WebService;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;
import javax.persistence.Query;

import org.apache.log4j.Logger;
import org.jboss.ws.annotation.PortComponent;

import de.bsd.weblog.interceptors.TimingInterceptor;
import de.bsd.weblog.interfaces.Facade;
import de.bsd.weblog.persistence.Artikel;
import de.bsd.weblog.persistence.NRolle;
import de.bsd.weblog.persistence.Nutzer;
import de.bsd.weblog.persistence.PLevel;
import de.bsd.weblog.persistence.Poster;
import de.bsd.weblog.persistence.Weblog;

/**
 * Facade, um auf Eintrge zuzugreifen
 * @author hwr@pilhuhn.de
 * @version $Revision: 1.4 $
 * 
 */

// JBoss-spezifisch. Legt den Webservice + wsdl nach
// http://lilly:8080/weblogws/FacadeSessionBean?wsdl
@PortComponent(contextRoot="/weblogws")

// Export des Beans als Webservice
@WebService(targetNamespace="http://bsd.de/weblog/")
@PermitAll
@Stateless
@Interceptors(TimingInterceptor.class)
public class FacadeSessionBean implements Facade
{
   private static final String DEFAULT_BLOG = "default";


   @PersistenceContext
   private EntityManager em;
   
   Logger log = Logger.getLogger(FacadeSessionBean.class);

	
	/* (non-Javadoc)
	 * @see de.bsd.weblog.ejb.Facade#countArticles(java.lang.String)
	 */
   @WebMethod
	public int anzahlArtikelInBlog(String blog) 
	{
		Query q = em.createNamedQuery("ArtikelInWeblog");
		q.setParameter("weblog", blog);
		Object o = q.getSingleResult();
		return ((Long)o).intValue();
	}
	
	/* (non-Javadoc)
	 * @see de.bsd.weblog.ejb.Facade#getArticleById(java.lang.String)
	 */
	public Artikel holeArtikelNachId(String id) {
		long idl = new Long(id).longValue();
		
		Artikel al = em.find(Artikel.class,idl);
		return al;		
	}
	
	@WebMethod
	public String holeArtikelTextNachId(String id)
	{
		long idl = new Long(id).longValue();
		
		Artikel al = em.find(Artikel.class,idl);
		return al.getText();
	}
	
	/** 
	 * Liest die ersten 100 Bytes eines Artikel-Bodys
	 * @see de.bsd.weblog.interfaces.Facade#holeArtikelExzerpt(java.lang.String)
	 */
	public String holeArtikelExzerpt(String artId) {
		Long idl = new Long(artId);
		Query q =  em.createNamedQuery("EinExzerpt");
		q.setParameter(1, idl);
		String ret = (String)q.getSingleResult();
		
		return ret;
	}

	/* (non-Javadoc)
	 * @see de.bsd.weblog.ejb.Facade#countArticlesByYear(java.lang.String, int)
	 */
	public int anzahlArtikelProJahr(String blog,int year) 
	{

		// Start- und Enddatum berechnen
		Calendar cal = setUpCalendar(year);
		Date startDate = cal.getTime();
		cal.set(Calendar.YEAR,year +1 );
		Date endDate = cal.getTime();
		
		
		Query q = em.createNamedQuery("ArtikelProJahr"); 
		q.setParameter("weblog", blog);
		q.setParameter("start", startDate);
		q.setParameter("end", endDate);
		Object o = q.getSingleResult();
		return ((Long)o).intValue();
		
	}


   /*
	 * (non-Javadoc)
	 * 
	 * @see de.bsd.weblog.ejb.Facade#getAll(java.lang.String)
	 */
	@SuppressWarnings("unchecked")
	public Collection<Artikel> holeAlleArtikelFuerBlog(String blog) {

		Collection<Artikel> co;

		Weblog w = weblogNachName(blog);
		if (w==null)
			return null;
		
		// Primrschlssel lesen
		Query q = em.createQuery("SELECT a FROM Artikel AS a JOIN FETCH a.poster WHERE "
				+ "a.weblog=:blog ORDER BY a.datum DESC, a.id DESC");
		q.setParameter("blog", w);
		co = q.getResultList();

		int len = co.size();
		System.out.println(">> getAll() found " + len + " Articles for " + blog);
		
		return co;
	}
   


	@SuppressWarnings("unchecked")
	public Collection<Artikel> holeArtikelFuerBlog(String blog, int abPosition, int anzahl) {
		Query q = em.createQuery("SELECT a FROM Artikel a WHERE a.weblog.name=:blog" +
				"ORDER BY a.id DESC");
		q.setParameter("blog", blog);
		q.setFirstResult(abPosition);
		q.setMaxResults(anzahl);
		
		Collection<Artikel> ret = q.getResultList();
		return ret;
	}

	/**
	 * Listet alle Blogs. Die Namen werden aufteigend sortiert.
	 * @see de.bsd.weblog.ejb.Facade#getAllBlogs()
	 */
	@SuppressWarnings("unchecked")
	public Collection<Weblog> listeAlleBlogs() 
	{
		
		Query q = em.createQuery("SELECT w FROM Weblog AS w ORDER BY w.name ASC");
		List co = q.getResultList();
		return co;
	}

   /** 
    * Fuegt einen Artikel zu einem Blog hinzu
    * @see de.bsd.weblog.ejb.Facade#addOne(de.bsd.weblog.persistence.Article, java.lang.String, java.lang.String)
    * @param a Der neue Artikel
    * @param poster E-Mail des Posters (=PK)
    * @param weblog Name des Weblogs in das der Artikel eingestellt werden soll
    */
	@WebMethod
   public Long artikelHinzufuegen(Artikel a,String posterId,String weblog) 
   {
	   Weblog w = weblogNachName(weblog);
	   Poster p = em.getReference(Poster.class,posterId);
	   Long ret = artikelHinzufuegen(a, p, w);
	   return ret;
   }
 
	/**
	 * Fgt einen neuen Artikel zum Blog hinzu
    * @param art Der neue Artikel
    * @param posterName Name des Posters (!=PK)
    * @param weblog Name des Weblogs in das der Artikel eingestellt werden soll
	 * @return Id des neuen Artikels
	 */
	public Long artikelHinzufuegenPN(Artikel art, String posterName, String weblog)
	{
		Weblog w = weblogNachName(weblog);
		Poster p = posterNachName(posterName);
		Long ret = artikelHinzufuegen(art, p, w);
		return ret;		
	}

   
   /**
	 * Fgt einen Artikel zu einem Blog hinzu
	 * @param art  Der neue Artikel
	 * @param post Der Poster
	 * @param blog Das Blog in das der Artikel eingestellt werden soll.
	 * 
	 * @see de.bsd.weblog.interfaces.Facade#artikelHinzufuegen(de.bsd.weblog.persistence.Artikel,
	 *      de.bsd.weblog.persistence.Poster, de.bsd.weblog.persistence.Weblog)
	 */
	public Long artikelHinzufuegen(Artikel art, Poster post, Weblog blog) {
		
		if (post == null || blog == null) {
			log.info("artikelHinzufuegen: poster oder blog ist null");
			return null;
		}
		
		if (art == null) {
			log.info("artikelHinzufuegen: artikel ist null");
			return null;
		}
		
		art.setPoster(post);
		post.addArticle(art);
		art.setWeblog(blog);
		blog.getArtikel().add(art);

		em.persist(art);
//		em.merge(post);
//		em.merge(blog);
		
		return art.getId();
	}

/* (non-Javadoc)
 * @see de.bsd.weblog.ejb.Facade#isPosterForBlog(java.lang.String, java.lang.String)
 */
   public boolean istPosterFuerBlog(String poster,String blog) {

	   // Funktioniert nicht, da unidirektionale Relation
//	   Query q = em.createQuery("SELECT p FROM Poster AS p WHERE " +
//			   "p.name = :poster AND p.weblog.name = :blog");
	   
	   log.info("istPosterFuerBlog: poster= " + poster + ", blog= " + blog);
	   
	   Query q = em.createQuery("SELECT w FROM Weblog AS w, Poster AS p WHERE " +
			   "w.name= :blog AND p.name = :poster AND p MEMBER OF w.nutzer AND p.rolle = 'POSTER'");
	   q.setParameter("poster", poster);
	   q.setParameter("blog", blog);
	   List l = q.getResultList();
	   boolean ret;
	   
	   if (l.size()==1)
		   ret = true;
	   else
		   ret =  false;
	   return ret;
   }

   /*
	 * (non-Javadoc)
	 * 
	 * @see de.bsd.weblog.ejb.Facade#initPoster()
	 */
	public void initPoster() {

		log.info("initPoster()");
		
		Weblog wl = weblogNachName(DEFAULT_BLOG);
		if (wl == null) {
			wl = new Weblog(DEFAULT_BLOG);
			em.persist(wl);
		}
		
		Poster pl = em.find(Poster.class, "hwr@pilhuhn.de");

		if (pl == null) {
			log.info("Poster 'hwr@pilhuhn.de' noch nicht angelegt");
			try {
				/*
				 * hwr ist ein Poster fr das Blog "default"
				 */
				Poster hwr = new Poster("hwr@pilhuhn.de", "hwr");
				hwr.setPasswort("geheim");
				hwr.setLevel(PLevel.EXPERTE);
				hwr.setRolle(NRolle.POSTER);
				
				hwr.setGeekCode("a+++ b--- c++ d** e-#+ f{} h}{");

				// add hwr to the posters of "default"
				Collection<Nutzer> co2 = wl.getNutzer();
				co2.add(hwr);
				wl.setNutzer(co2);

				/*
				 * Admin hinzufgen, aber ohne Weblog
				 */
				Poster admin = new Poster("hwr@bsd.de", "admin");
				admin.setPasswort("admin");
				admin.setRolle(NRolle.ADMIN);

				em.persist(hwr);
//				em.merge(wl);
//				em.persist(admin);

			} catch (PersistenceException ce) {
				System.err.println("Create of initial user failed!!");
				return;
			}
		} else {
			log.info("User hwr@pilhuhn.de schon da -- nichts zu tun.");
		}
	}
   
   /**
	 * Ermittelt das zum gegebenen Namen gehrende Blog
	 * @param weblog Name des Blogs
	 * @return Das Blog oder null, falls nicht gefunden.
	 */
	protected Weblog weblogNachName(String weblog) {
		Query q = em.createNamedQuery("WeblogNachName");
		q.setParameter("blog",weblog);
		Weblog wl = null;
		try {
			wl = (Weblog)q.getSingleResult();
		}
		catch (NoResultException nre) {
			if (log.isDebugEnabled())
				log.debug("weblogNachName: " + weblog + " nicht gefunden");
		}
		return wl;
	}

	/**
	 * Ermittelt den Poster zum gegebenen Namen
	 * @param name Name des Posters
	 * @return Der Poster oder null, falls nicht gefunden.
	 */
	protected Poster posterNachName(String name) {
		Query q = em.createNamedQuery("PosterNachName");
		q.setParameter("name", name);
		Poster ret = null;
		try {
			ret = (Poster)q.getSingleResult(); 
		}
		catch (NoResultException nre) {
			if (log.isDebugEnabled())
				log.debug("posterNachName: " + name + " nicht gefunden");
		}
		return ret;
	}
	
	/**
	 * Legt ein Calendar an, bei dem das Datum auf die erste Sekunde im 
	 * Jahr gesetzt ist.
	 * @param year Das gewnschte Jahr
	 * @return Einen initalisierten Calendar.
	 * @see java.util.Calendar
	 */
	private Calendar setUpCalendar(int year) {
		Calendar start = Calendar.getInstance();
		start.set(Calendar.DAY_OF_MONTH, 1);
		start.set(Calendar.MONTH,Calendar.JANUARY);
		start.set(Calendar.YEAR,year);
		start.set(Calendar.HOUR_OF_DAY,0);
		start.set(Calendar.MINUTE,0);
		start.set(Calendar.SECOND, 0);
		start.set(Calendar.MILLISECOND,0);
		return start;
	}

   
}
