/*
 * Created on 09.06.2006
 * $Id: Facade.java,v 1.2 2006/10/22 16:49:24 hwr Exp $
 */
package de.bsd.weblog.interfaces;


import java.util.Collection;

import javax.ejb.Remote;

import de.bsd.weblog.persistence.Artikel;
import de.bsd.weblog.persistence.Poster;
import de.bsd.weblog.persistence.Weblog;

@Remote
public interface Facade {

	/**
	 * Zhlt die Anzahl der Artikel in einem Blog
	 * @param blog Name des Blogs
	 * @return Anzahl der Artikel im Blog
	 */
	public int anzahlArtikelInBlog(String blog);

	/**
	 * Return the article identified by id
	 * @param id Basically the primary key of the article
	 * @return the whole Article
	 */
	public Artikel holeArtikelNachId(String id);

	/**
	 * Liest die ersten 100 Bytes eines Artikel-Bodys
	 * @param artId der PK des Artikels
	 * @return die ersten 100 Bytes
	 */
	public String holeArtikelExzerpt(String artId);
	/**
	 * Count the articles of the specified year
	 * @param blog name of the blog
	 * @param year for which year do we search?
	 * @return number of articles found for that blog
	 */
	public int anzahlArtikelProJahr(String blog, int year);

	/**
	 * Returns all stored articles
	 * @return
	 */
	public Collection<Artikel> holeAlleArtikelFuerBlog(String blog);

	/**
	 * Holt eine gewisse Anzahl an Artikeln aus dem Blog
	 * @param blog Name des Blogs
	 * @param abPosition Position in der Trefferliste (0=neuester)
	 * @param anzahl Anzahl der zu lesenden Artikel
	 * @return
	 */
	public Collection<Artikel> holeArtikelFuerBlog(String blog, int abPosition, int anzahl);
	
	/**
	 * Listet die Namen aller Blogs im System
	 * @return
	 */
	public Collection<Weblog> listeAlleBlogs();

	/**
	 * Fgt einen neuen Artikel zum Blog hinzu
     * @param art Der neue Artikel
     * @param posterId E-Mail des Posters (=PK)
     * @param weblog Name des Weblogs in das der Artikel eingestellt werden soll
	 * @return Id des neuen Artikels
 	 */
	public Long artikelHinzufuegen(Artikel art, String posterId, String weblog);

	/**
	 * Fgt einen neuen Artikel zum Blog hinzu
     * @param art Der neue Artikel
     * @param posterName Name des Posters (!=PK)
     * @param weblog Name des Weblogs in das der Artikel eingestellt werden soll
	 * @return Id des neuen Artikels
 	 */
	public Long artikelHinzufuegenPN(Artikel art, String posterName, String weblog);
	
	/**
	 * Fgt einen neuen Artikel zum Blog hinzu
	 * @param art Der neue Artikel
	 * @param post Der Poster
	 * @param blog Das Blog in das der Artikel eingestellt werden soll.
	 * @return Id des neuen Artikels
	 */
	public Long artikelHinzufuegen(Artikel art, Poster post, Weblog blog);

	/**
	 * Darf der bergebene Benutzer in das Blog schreiben?
	 * @param poster Der Benutzer
	 * @param blog Das gewhlte Blog
	 */
	public boolean istPosterFuerBlog(String poster, String blog);

	/**
	 * Initialisiert die Poster, falls sie noch nicht in der Datenbank
	 * sind.
	 */
	public void initPoster();

}