/*
 * Created on 29.12.2003
 *
 * (c) 2002,2003 Cellent AG Finance Solutions
 * 
 * $Id: Artikel.java,v 1.2 2006/10/19 20:23:15 hwr Exp $
 */
package de.bsd.weblog.persistence;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.ColumnResult;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.NamedNativeQuery;
import javax.persistence.NamedQuery;
import javax.persistence.SqlResultSetMapping;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Version;



/**
 * Ein Artikel.
 * @author hwr@pilhuhn.de
 * @version $Revision: 1.2 $
 */
// Wird von JBoss in EJB-3-fd-cr9 nicht untersttzt
//@NamedNativeQuery(name="EinExzerpt",
//		query="SELECT substring(text for 100) FROM WL_Article WHERE id = ?1")

@NamedNativeQuery(name="EinExcerpt",
		resultSetMapping="toString",
		query="SELECT substring(text for 100) AS start  FROM WL_Artikel WHERE id = ?1")
@SqlResultSetMapping(name="toString",
		columns=@ColumnResult(name="start"))

@NamedQuery(name="ArtikelProJahr",
		query="SELECT COUNT(*) FROM Artikel AS a " 
				+"WHERE a.weblog.name = :weblog AND a.datum >= :start AND a.datum < :end"
)
@Entity
@Table(name="WL_Artikel")
public class Artikel implements Serializable
{
	private static final long serialVersionUID = 1L;
	private Long id;      // Primrschlssel
	private Date datum;   // Datum des Postings
	private String titel; 
	private Poster poster;
	private Weblog weblog;
	private String text;  // Body-Text
	private long version;

	public Artikel()
	{
		
	}
		
	/**
	 * Das Datum des Artikels
	 */
	@Temporal(TemporalType.DATE)
	public Date getDatum() {
		return datum;
	}
	

	public void setDatum(Date date) {
		this.datum = date;
	}	

	/**
	 * PrimaryKey of this table
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	public Long getId() {
		return id;
	}
	
	/** 
	 * Setter fr den Primrschlssel. Dieser ist privat,
	 * da dieser generiert wird.
	 */
	@SuppressWarnings("unused")
	private void setId(Long id)
	{
		this.id = id;
	}
	

	/**
	 * Der Titel des Artikels
	 */
	public  String getTitel() {
		return titel;
	}
	
	public void setTitel(String subject)
	{
		this.titel = subject;
	}

	/**
	 * Text des Artikels
	 */
	@Lob
	public String getText() {
		return text;
	}
	
	public void setText(String text)
	{
		this.text = text;
	}
	
	
	/**
	 * Der Poster des Eintrags
	 */
	@ManyToOne
	public Poster getPoster()
	{
		return poster;
	}
	
	public void setPoster(Poster poster)
	{
		this.poster = poster;
	}
	
	/**
	 * Ein Weblog kann viele Artikel enthalten
	 */
	@ManyToOne
	public Weblog getWeblog() {
		return weblog;
	}
	
	public void setWeblog(Weblog weblog)
	{
		this.weblog = weblog;
	}


	/**
	 * Versionsspalte fr das optimistic Locking
	 */
	@Version
	public long getVersion() {
		return version;
	}

	@SuppressWarnings("unused")
	private void setVersion(long version) {
		this.version = version;
	}
}