/*
 * (c) hwr@pilhuhn.de 
 * $Id: Poster.java,v 1.3 2006/10/22 16:49:24 hwr Exp $
 */
package de.bsd.weblog.persistence;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.Lob;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.SecondaryTable;

/**
 * Ein Poster ist eine Person, die Artikel schreiben darf.
 * @author hwr@pilhuhn.de
 * @version $Revision: 1.3 $
 * 
 */
@NamedQueries({
	@NamedQuery(name="ArtikelDesPostersMitName",
		query="SELECT a FROM Artikel AS a WHERE a.poster.name = :pname"),
	@NamedQuery(name="ArtikelDesPosters",
		query="SELECT a FROM Artikel AS a WHERE a.poster = :poster"),
	@NamedQuery(name="Poster.PosterListe",
		query="SELECT p FROM Poster p"),
	@NamedQuery(name="PosterNachName",
		query="SELECT p FROM Poster p WHERE p.name = :name")	
})
@SecondaryTable(name="WL_PExtra")
@Entity
public class Poster extends Nutzer implements Serializable 
{

	/**
	 * Generierte SVUID
	 */
	private static final long serialVersionUID = -6126751821564740029L;
	
	private String name;
	private PLevel level;
	private Collection<Artikel> artikel = new ArrayList<Artikel>();

	private String geekCode;

	/**
	 * For JPA use
	 */
	protected Poster()
	{
		setRolle(NRolle.GAST); // Vorbelegung fr JAAS
		setLevel(PLevel.NEU);
	}
	
	public Poster(String email, String login)
	{
		this(email, login, PLevel.NEU);
	}
	
	public Poster(String email, String login, PLevel level) 
	{
		super(email);
		setName(login);
		setLevel(level);
	}
	
	
	/**
	 * One poster can post several artikel, but 
	 * an article only belongs to one poster.
	 *  
	 * @return List of artikel by this poster.
	 */
	@OneToMany(mappedBy="poster")
	public Collection<Artikel> getArtikel()
	{
		return artikel;
	}
	
	/** 
	 */
	public void setArtikel(Collection<Artikel> articles)
	{
		this.artikel = articles;
	}
	
	public Collection<Artikel> addArticle(Artikel art)
	{
		Collection<Artikel> col = getArtikel();
		art.setPoster(this);
		col.add(art);
		setArtikel(col);
		return col;
	}

	/**
	 * Nickname des Posters. Muss auch
	 * eindeutig sein ber alle Weblogs hinweg.
	 * @param name
	 */
	@Column(length=16, unique=true)
	public String getName() {
		return name;
	}

	private void setName(String loginName) {
		this.name = loginName;
	}


	/**
	 * Erfahrungslevel des Nutzers
	 * @return
	 */
	@Enumerated(EnumType.ORDINAL)
	public PLevel getLevel() {
		return level;
	}

	public void setLevel(PLevel level) {
		this.level = level;
	}

	/**
	 * Der Geek Code des Posters
	 * Liegt in einer separaten Tabelle und soll
	 * nur bei Bedarf geladen werden.
	 * @return the code
	 */
	@Lob
	@Column(table="WL_PExtra")
	@Basic(fetch=FetchType.LAZY)
	public String getGeekCode() {
		return geekCode;
	}

	/**
	 * @param code der GeekCode des Posters
	 */
	public void setGeekCode(String code) {
		geekCode = code;
	}
	

	
}
