/*
 * $Id: Weblog.java,v 1.2 2006/10/19 20:23:15 hwr Exp $
 */
package de.bsd.weblog.persistence;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashSet;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Version;


/**
 * Die einzelnen Blogs im System
 * 
 * @author hwr@pilhuhn.de
 * @version $Revision: 1.2 $
 */
@NamedQueries({
	@NamedQuery(name="WeblogNachName",
		query="SELECT w FROM Weblog AS w WHERE "+
				"w.name = :blog"),
	@NamedQuery(name="ArtikelInWeblog",
		query="SELECT COUNT(*) FROM Artikel AS a "+
				"WHERE a.weblog.name = :weblog")
})
			
@Entity
@Table(name="WL_WEBLOGS")
public class Weblog implements Serializable
{

	private static final long serialVersionUID = 1L;
	
	private Long id;     // Primrschlssel
	private String name; // Name des Blogs
	private Collection<Artikel> artikel = 
		new HashSet<Artikel>(); // Artikel dieses Blogs
	private Collection<Nutzer> nutzer = 
		new HashSet<Nutzer>();  // Poster und Nutzer 
	private long version; // Version fr Optimisic Locking
	
	/**
	 * JPA bentigt einen nicht privaten Konstruktor
	 *
	 */
	protected Weblog()
	{
		
	}

	public Weblog(String name)
	{
		this.name = name;
	}

	/**
	 * Der Primrschlssel
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	public Long getId() {
		return id;
	}


	/**
	 * Primrschlssel wird nur von JPA gesetzt.
	 * @param id
	 */
	@SuppressWarnings("unused")
	private void setId(Long id) {
		this.id = id;
	}

	/**
	 * Der Name des Blogs.
	 * Dieser muss eindeutig sein.
	 * @return
	 */
	@Column(unique=true, length=64)
	public String getName()
	{
		return name;
	}
	
	public void setName(String wname)
	{
		name = wname;
	}
	
	
	/**
	 * Ein Weblog kann mehrere Nutzer haben.
	 * @return Die Nutzer dieses Blogs
	 */
	@JoinTable(name="WL_Weblog_Nutzer")
	@ManyToMany
	public Collection<Nutzer> getNutzer()
	{
		return nutzer;
	}
	
	public void setNutzer(Collection<Nutzer> posters)
	{
		this.nutzer = posters;
	}
	
	/**
	 * Ein Weblog kann viele Artikel enthalten.
	 */
	@OneToMany(mappedBy="weblog", 
			cascade=CascadeType.REMOVE)
//	@JoinColumn(name="wl_art_fk", referencedColumnName="id" ) // Bug?
	public Collection<Artikel> getArtikel()
	{
		return artikel;
	}
	
	public void setArtikel(Collection<Artikel> articles)
	{
		this.artikel = articles;
	}

	/**
	 * Versionierungsspalte fr Optimistic Locking 
	 */
	@Version
	public long getVersion() {
		return version;
	}

	@SuppressWarnings("unused")
	private void setVersion(long version) {
		this.version = version;
	}

}
