/*
 * 
 * $Id: DisplayServlet.java,v 1.2 2006/10/22 16:49:23 hwr Exp $
 */
package de.bsd.weblog.servlets;

import java.io.IOException;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;

import javax.ejb.EJB;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import de.bsd.weblog.interfaces.Facade;
import de.bsd.weblog.persistence.Artikel;
import de.bsd.weblog.persistence.Weblog;

/**
 * @author hwr@pilhuhn.de
 * @version $Revision: 1.2 $
 */
@SuppressWarnings("serial")
public class DisplayServlet extends HttpServlet {

	InitialContext ic;
	@EJB
	Facade fa;
	
	/* (non-Javadoc)
	 * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
		throws ServletException, IOException {

		String path =
			req.getAttribute("javax.servlet.include.path_info").toString();
		System.out.println("path: " + path );

		if (path.charAt(0) == '/')
			path = path.substring(1);

		String anId = null;
		if (path.startsWith("_perm")) {
			anId = path.substring(path.indexOf("=") + 1);
		}

		PrintWriter out = resp.getWriter();
		if (ic == null ) {
			resp.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
			out.println("<h1>Interner Fehler</h1>");
			return;
		}

		try {
			// list blogs
			Collection<Weblog> blogs = fa.listeAlleBlogs();
			if (blogs != null) {
				out.println("<div class=\"small\">");
				out.print("Blogs: ");
				for (Weblog w: blogs) {
					out.println(
						" [ <a href=\"index.jsp?blog="
							+ w.getName()
							+ "\" class=\"button\">"
							+ w.getName()
							+ "</a> ] ");
				}
				out.println("</div><p/>");
			} else {
				System.err.println("Keine blogs!");
			}

			// Artikel fuer das Blog anzeigen.
			Collection<Artikel> as = null;
			if (anId != null) { // a single article
				Artikel art = fa.holeArtikelNachId(anId);
				as = new ArrayList<Artikel>();
				as.add(art);
			} else {
				as = fa.holeAlleArtikelFuerBlog(path);
			}

			out.println("<table class=\"indent-small\">");
			int i = 0;
			for (Artikel art : as) {
				if (i==0)
					out.println(" <tr class=\"yellow\">");
				else 
					out.println(" <tr class=\"whiteback\">");
				i = 1-i;
				
				DateFormat df = DateFormat.getDateInstance(DateFormat.MEDIUM, Locale.GERMANY);
				String datString = df.format(art.getDatum());
				
				out.println(
							"  <td class=\"whiteback\">"							
								+ "<span style=\"small\">"
								+ art.getPoster().getName()
								+ "<br/>"
								+ datString
								+ "</span>"
								+ "</td>");
					getServletContext().setAttribute("art", art);
					RequestDispatcher rd =
						getServletContext().getRequestDispatcher(
							"/displayOneArticle.jsp");
					rd.include(req, resp);
					out.println(" </tr>");
			}
			out.println("</table>");
		} catch (Exception e) {
			out.println("Error: " + e.getMessage());
			e.printStackTrace(out);
		}
	}

	/* (non-Javadoc)
	 * @see javax.servlet.Servlet#init(javax.servlet.ServletConfig)
	 */
	public void init(ServletConfig arg0) throws ServletException {
		super.init(arg0);
		try {
			ic = new InitialContext();
		} catch (NamingException e) {
			System.err.println(
				"Error obtaining Initial Context: " + e.getMessage());
			ic = null;
		}

		if (fa==null) { // Injection failed
			try {
				// weblog-Prefix wegen Deployment im EAR
				fa = (Facade) ic.lookup("weblog/FacadeSessionBean/remote");
			} catch (Exception e) {
				System.err.println("Error obtaining Facade: " + e.getMessage());
			}
		}
		try {
			fa.initPoster();
			Artikel a = new Artikel();
			GregorianCalendar gc = new GregorianCalendar();
			gc.setTime(new Date()); // now 

			if (false) {
				a.setText("Dies ist ein Test. Dieser Satz kein Verb. The quick brown fox jumps over the lazy dog.");
				fa.artikelHinzufuegen(a, "hwr", "default");
			} // if(true/false)

		} catch (Exception e) {
			System.err.println("Can't add article: " + e.getMessage());
		}

	}
}
